
# 'iso_639_1', 'language_name'
LANGUAGES = (('ab', 'Abkhazian'), ('aa', 'Afar'), ('af', 'Afrikaans'), ('ak', 'Akan'),
             ('sq', 'Albanian'), ('am', 'Amharic'), ('ar', 'Arabic'), ('an', 'Aragonese'), ('hy', 'Armenian'),
             ('as', 'Assamese'), ('av', 'Avaric'), ('ae', 'Avestan'), ('ay', 'Aymara'), ('az', 'Azerbaijani'),
             ('bm', 'Bambara'), ('ba', 'Bashkir'), ('eu', 'Basque'), ('be', 'Belarusian'), ('bn', 'Bengali'),
             ('bh', 'Bihari languages'), ('bi', 'Bislama'), ('bs', 'Bosnian'), ('br', 'Breton'), ('bg', 'Bulgarian'),
             ('my', 'Burmese'), ('ca', 'Catalan, Valencian'), ('ch', 'Chamorro'), ('ce', 'Chechen'),
             ('ny', 'Chichewa, Chewa, Nyanja'), ('zh', 'Chinese'), ('cv', 'Chuvash'), ('kw', 'Cornish'),
             ('co', 'Corsican'), ('cr', 'Cree'), ('hr', 'Croatian'), ('cs', 'Czech'), ('da', 'Danish'),
             ('dv', 'Divehi, Dhivehi, Maldivian'), ('nl', 'Dutch, Flemish'), ('dz', 'Dzongkha'), ('en', 'English'),
             ('eo', 'Esperanto'), ('et', 'Estonian'), ('ee', 'Ewe'), ('fo', 'Faroese'), ('fj', 'Fijian'),
             ('fi', 'Finnish'), ('fr', 'French'), ('ff', 'Fulah'), ('gl', 'Galician'), ('ka', 'Georgian'),
             ('de', 'German'), ('el', 'Greek, Modern (1453–)'), ('gn', 'Guarani'), ('gu', 'Gujarati'),
             ('ht', 'Haitian, Haitian Creole'), ('ha', 'Hausa'), ('he', 'Hebrew'), ('hz', 'Herero'), ('hi', 'Hindi'),
             ('ho', 'Hiri Motu'), ('hu', 'Hungarian'),
             ('ia', 'Interlingua (International Auxiliary Language Association)'), ('id', 'Indonesian'),
             ('ie', 'Interlingue, Occidental'), ('ga', 'Irish'), ('ig', 'Igbo'), ('ik', 'Inupiaq'), ('io', 'Ido'),
             ('is', 'Icelandic'), ('it', 'Italian'), ('iu', 'Inuktitut'), ('ja', 'Japanese'), ('jv', 'Javanese'),
             ('kl', 'Kalaallisut, Greenlandic'), ('kn', 'Kannada'), ('kr', 'Kanuri'), ('ks', 'Kashmiri'),
             ('kk', 'Kazakh'), ('km', 'Central Khmer'), ('ki', 'Kikuyu, Gikuyu'), ('rw', 'Kinyarwanda'),
             ('ky', 'Kirghiz, Kyrgyz'), ('kv', 'Komi'), ('kg', 'Kongo'), ('ko', 'Korean'), ('ku', 'Kurdish'),
             ('kj', 'Kuanyama, Kwanyama'), ('la', 'Latin'), ('lb', 'Luxembourgish, Letzeburgesch'), ('lg', 'Ganda'),
             ('li', 'Limburgan, Limburger, Limburgish'), ('ln', 'Lingala'), ('lo', 'Lao'), ('lt', 'Lithuanian'),
             ('lu', 'Luba-Katanga'), ('lv', 'Latvian'), ('gv', 'Manx'), ('mk', 'Macedonian'), ('mg', 'Malagasy'),
             ('ms', 'Malay'), ('ml', 'Malayalam'), ('mt', 'Maltese'), ('mi', 'Maori'), ('mr', 'Marathi'),
             ('mh', 'Marshallese'), ('mn', 'Mongolian'), ('na', 'Nauru'), ('nv', 'Navajo, Navaho'),
             ('nd', 'North Ndebele'), ('ne', 'Nepali'), ('ng', 'Ndonga'), ('nb', 'Norwegian Bokmål'),
             ('nn', 'Norwegian Nynorsk'), ('no', 'Norwegian'), ('ii', 'Sichuan Yi, Nuosu'), ('nr', 'South Ndebele'),
             ('oc', 'Occitan'), ('oj', 'Ojibwa'),
             ('cu', 'Church Slavic, Old Slavonic, Church Slavonic, Old Bulgarian, Old Church Slavonic'),
             ('om', 'Oromo'), ('or', 'Oriya'), ('os', 'Ossetian, Ossetic'), ('pa', 'Punjabi, Panjabi'), ('pi', 'Pali'),
             ('fa', 'Persian'), ('pl', 'Polish'), ('ps', 'Pashto, Pushto'), ('pt', 'Portuguese'), ('qu', 'Quechua'),
             ('rm', 'Romansh'), ('rn', 'Rundi'), ('ro', 'Romanian, Moldavian, Moldovan'), ('ru', 'Russian'),
             ('sa', 'Sanskrit'), ('sc', 'Sardinian'), ('sd', 'Sindhi'), ('se', 'Northern Sami'), ('sm', 'Samoan'),
             ('sg', 'Sango'), ('sr', 'Serbian'), ('gd', 'Gaelic, Scottish Gaelic'), ('sn', 'Shona'),
             ('si', 'Sinhala, Sinhalese'), ('sk', 'Slovak'), ('sl', 'Slovenian'), ('so', 'Somali'),
             ('st', 'Southern Sotho'), ('es', 'Spanish, Castilian'), ('su', 'Sundanese'), ('sw', 'Swahili'),
             ('ss', 'Swati'), ('sv', 'Swedish'), ('ta', 'Tamil'), ('te', 'Telugu'), ('tg', 'Tajik'), ('th', 'Thai'),
             ('ti', 'Tigrinya'), ('bo', 'Tibetan'), ('tk', 'Turkmen'), ('tl', 'Tagalog'), ('tn', 'Tswana'),
             ('to', 'Tonga (Tonga Islands)'), ('tr', 'Turkish'), ('ts', 'Tsonga'), ('tt', 'Tatar'), ('tw', 'Twi'),
             ('ty', 'Tahitian'), ('ug', 'Uighur, Uyghur'), ('uk', 'Ukrainian'), ('ur', 'Urdu'), ('uz', 'Uzbek'),
             ('ve', 'Venda'), ('vi', 'Vietnamese'), ('vo', 'Volapük'), ('wa', 'Walloon'), ('cy', 'Welsh'),
             ('wo', 'Wolof'), ('fy', 'Western Frisian'), ('xh', 'Xhosa'), ('yi', 'Yiddish'), ('yo', 'Yoruba'),
             ('za', 'Zhuang, Chuang'))

# 'iso _639_1', 'native_name', 'language_name'
# LANGUAGES = (
#     ('ab', 'аҧсуа бызшәа, аҧсшәа', 'Abkhazian'),
#     ('aa', 'Afaraf', 'Afar'),
#     ('af', 'Afrikaans', 'Afrikaans'), ('ak', 'Akan', 'Akan'), ('sq', 'Shqip', 'Albanian'), ('am', 'አማርኛ', 'Amharic'),
#     ('ar', 'العربية', 'Arabic'), ('an', 'aragonés', 'Aragonese'), ('hy', 'Հայերեն', 'Armenian'),
#     ('as', 'অসমীয়া', 'Assamese'), ('av', 'авар мацӀ, магӀарул мацӀ', 'Avaric'), ('ae', 'avesta', 'Avestan'),
#     ('ay', 'aymar aru', 'Aymara'), ('az', 'azərbaycan dili', 'Azerbaijani'), ('bm', 'bamanankan', 'Bambara'),
#     ('ba', 'башҡорт теле', 'Bashkir'), ('eu', 'euskara, euskera', 'Basque'), ('be', 'беларуская мова', 'Belarusian'),
#     ('bn', 'বাংলা', 'Bengali'), ('bh', 'भोजपुरी', 'Bihari languages'), ('bi', 'Bislama', 'Bislama'),
#     ('bs', 'bosanski jezik', 'Bosnian'), ('br', 'brezhoneg', 'Breton'), ('bg', 'български език', 'Bulgarian'),
#     ('my', 'ဗမာစာ', 'Burmese'), ('ca', 'català, valencià', 'Catalan, Valencian'), ('ch', 'Chamoru', 'Chamorro'),
#     ('ce', 'нохчийн мотт', 'Chechen'), ('ny', 'chiCheŵa, chinyanja', 'Chichewa, Chewa, Nyanja'),
#     ('zh', '中文 (Zhōngwén), 汉语, 漢語', 'Chinese'), ('cv', 'чӑваш чӗлхи', 'Chuvash'), ('kw', 'Kernewek', 'Cornish'),
#     ('co', 'corsu, lingua corsa', 'Corsican'), ('cr', 'ᓀᐦᐃᔭᐍᐏᐣ', 'Cree'), ('hr', 'hrvatski jezik', 'Croatian'),
#     ('cs', 'čeština, český jazyk', 'Czech'), ('da', 'dansk', 'Danish'),
#     ('dv', 'ދިވެހި', 'Divehi, Dhivehi, Maldivian'),
#     ('nl', 'Nederlands, Vlaams', 'Dutch, Flemish'), ('dz', 'རྫོང་ཁ', 'Dzongkha'), ('en', 'English', 'English'),
#     ('eo', 'Esperanto', 'Esperanto'), ('et', 'eesti, eesti keel', 'Estonian'), ('ee', 'Eʋegbe', 'Ewe'),
#     ('fo', 'føroyskt', 'Faroese'), ('fj', 'vosa Vakaviti', 'Fijian'), ('fi', 'suomi, suomen kieli', 'Finnish'),
#     ('fr', 'français, langue française', 'French'), ('ff', 'Fulfulde, Pulaar, Pular', 'Fulah'),
#     ('gl', 'Galego', 'Galician'), ('ka', 'ქართული', 'Georgian'), ('de', 'Deutsch', 'German'),
#     ('el', 'ελληνικά', 'Greek, Modern (1453–)'), ('gn', 'Avañe\'ẽ', 'Guarani'), ('gu', 'ગુજરાતી', 'Gujarati'),
#     ('ht', 'Kreyòl ayisyen', 'Haitian, Haitian Creole'), ('ha', '(Hausa) هَوُسَ', 'Hausa'), ('he', 'עברית', 'Hebrew'),
#     ('hz', 'Otjiherero', 'Herero'), ('hi', 'हिन्दी, हिंदी', 'Hindi'), ('ho', 'Hiri Motu', 'Hiri Motu'),
#     ('hu', 'magyar', 'Hungarian'),
#     ('ia', 'Interlingua', 'Interlingua (International Auxiliary Language Association)'),
#     ('id', 'Bahasa Indonesia', 'Indonesian'),
#     ('ie', '(originally:) Occidental, (after WWII:) Interlingue', 'Interlingue, Occidental'),
#     ('ga', 'Gaeilge', 'Irish'),
#     ('ig', 'Asụsụ Igbo', 'Igbo'), ('ik', 'Iñupiaq, Iñupiatun', 'Inupiaq'), ('io', 'Ido', 'Ido'),
#     ('is', 'Íslenska', 'Icelandic'), ('it', 'Italiano', 'Italian'), ('iu', 'ᐃᓄᒃᑎᑐᑦ', 'Inuktitut'),
#     ('ja', '日本語 (にほんご)', 'Japanese'), ('jv', 'ꦧꦱꦗꦮ, Basa Jawa', 'Javanese'),
#     ('kl', 'kalaallisut, kalaallit oqaasii', 'Kalaallisut, Greenlandic'), ('kn', 'ಕನ್ನಡ', 'Kannada'),
#     ('kr', 'Kanuri', 'Kanuri'), ('ks', 'कश्मीरी, كشميري', 'Kashmiri'), ('kk', 'қазақ тілі', 'Kazakh'),
#     ('km', 'ខ្មែរ, ខេមរភាសា, ភាសាខ្មែរ', 'Central Khmer'), ('ki', 'Gĩkũyũ', 'Kikuyu, Gikuyu'),
#     ('rw', 'Ikinyarwanda', 'Kinyarwanda'), ('ky', 'Кыргызча, Кыргыз тили', 'Kirghiz, Kyrgyz'),
#     ('kv', 'коми кыв', 'Komi'),
#     ('kg', 'Kikongo', 'Kongo'), ('ko', '한국어', 'Korean'), ('ku', 'Kurdî, کوردی', 'Kurdish'),
#     ('kj', 'Kuanyama', 'Kuanyama, Kwanyama'), ('la', 'latine, lingua latina', 'Latin'),
#     ('lb', 'Lëtzebuergesch', 'Luxembourgish, Letzeburgesch'), ('lg', 'Luganda', 'Ganda'),
#     ('li', 'Limburgs', 'Limburgan, Limburger, Limburgish'), ('ln', 'Lingála', 'Lingala'), ('lo', 'ພາສາລາວ', 'Lao'),
#     ('lt', 'lietuvių kalba', 'Lithuanian'), ('lu', 'Kiluba', 'Luba-Katanga'), ('lv', 'latviešu valoda', 'Latvian'),
#     ('gv', 'Gaelg, Gailck', 'Manx'), ('mk', 'македонски јазик', 'Macedonian'), ('mg', 'fiteny malagasy', 'Malagasy'),
#     ('ms', 'Bahasa Melayu, بهاس ملايو', 'Malay'), ('ml', 'മലയാളം', 'Malayalam'), ('mt', 'Malti', 'Maltese'),
#     ('mi', 'te reo Māori', 'Maori'), ('mr', 'मराठी', 'Marathi'), ('mh', 'Kajin M̧ajeļ', 'Marshallese'),
#     ('mn', 'Монгол хэл', 'Mongolian'), ('na', 'Dorerin Naoero', 'Nauru'), ('nv', 'Diné bizaad', 'Navajo, Navaho'),
#     ('nd', 'isiNdebele', 'North Ndebele'), ('ne', 'नेपाली', 'Nepali'), ('ng', 'Owambo', 'Ndonga'),
#     ('nb', 'Norsk Bokmål', 'Norwegian Bokmål'), ('nn', 'Norsk Nynorsk', 'Norwegian Nynorsk'),
#     ('no', 'Norsk', 'Norwegian'),
#     ('ii', 'ꆈꌠ꒿ Nuosuhxop', 'Sichuan Yi, Nuosu'), ('nr', 'isiNdebele', 'South Ndebele'),
#     ('oc', 'occitan, lenga d\'òc', 'Occitan'), ('oj', 'ᐊᓂᔑᓈᐯᒧᐎᓐ', 'Ojibwa'),
#     ('cu', 'ѩзыкъ словѣньскъ', 'Church Slavic, Old Slavonic, Church Slavonic, Old Bulgarian, Old Church Slavonic'),
#     ('om', 'Afaan Oromoo', 'Oromo'), ('or', 'ଓଡ଼ିଆ', 'Oriya'), ('os', 'ирон æвзаг', 'Ossetian, Ossetic'),
#     ('pa', 'ਪੰਜਾਬੀ, پنجابی', 'Punjabi, Panjabi'), ('pi', 'पालि, पाळि', 'Pali'), ('fa', 'فارسی', 'Persian'),
#     ('pl', 'język polski, polszczyzna', 'Polish'), ('ps', 'پښتو', 'Pashto, Pushto'),
#     ('pt', 'Português', 'Portuguese'),
#     ('qu', 'Runa Simi, Kichwa', 'Quechua'), ('rm', 'Rumantsch Grischun', 'Romansh'), ('rn', 'Ikirundi', 'Rundi'),
#     ('ro', 'Română', 'Romanian, Moldavian, Moldovan'), ('ru', 'русский', 'Russian'), ('sa', 'संस्कृतम्', 'Sanskrit'),
#     ('sc', 'sardu', 'Sardinian'), ('sd', 'सिन्धी, سنڌي، سندھی', 'Sindhi'), ('se', 'Davvisámegiella', 'Northern Sami'),
#     ('sm', 'gagana fa\'a Samoa', 'Samoan'), ('sg', 'yângâ tî sängö', 'Sango'), ('sr', 'српски језик', 'Serbian'),
#     ('gd', 'Gàidhlig', 'Gaelic, Scottish Gaelic'), ('sn', 'chiShona', 'Shona'), ('si', 'සිංහල', 'Sinhala, Sinhalese'),
#     ('sk', 'Slovenčina, Slovenský Jazyk', 'Slovak'), ('sl', 'Slovenski Jezik, Slovenščina', 'Slovenian'),
#     ('so', 'Soomaaliga, af Soomaali', 'Somali'), ('st', 'Sesotho', 'Southern Sotho'),
#     ('es', 'Español', 'Spanish, Castilian'), ('su', 'Basa Sunda', 'Sundanese'), ('sw', 'Kiswahili', 'Swahili'),
#     ('ss', 'SiSwati', 'Swati'), ('sv', 'Svenska', 'Swedish'), ('ta', 'தமிழ்', 'Tamil'), ('te', 'తెలుగు', 'Telugu'),
#     ('tg', 'тоҷикӣ, toçikī, تاجیکی', 'Tajik'), ('th', 'ไทย', 'Thai'), ('ti', 'ትግርኛ', 'Tigrinya'),
#     ('bo', 'བོད་ཡིག', 'Tibetan'), ('tk', 'Türkmen, Түркмен', 'Turkmen'), ('tl', 'Wikang Tagalog', 'Tagalog'),
#     ('tn', 'Setswana', 'Tswana'), ('to', 'Faka Tonga', 'Tonga (Tonga Islands)'), ('tr', 'Türkçe', 'Turkish'),
#     ('ts', 'Xitsonga', 'Tsonga'), ('tt', 'татар теле, tatar tele', 'Tatar'), ('tw', 'Twi', 'Twi'),
#     ('ty', 'Reo Tahiti', 'Tahitian'), ('ug', 'ئۇيغۇرچە, Uyghurche', 'Uighur, Uyghur'),
#     ('uk', 'Українська', 'Ukrainian'),
#     ('ur', 'اردو', 'Urdu'), ('uz', 'Oʻzbek, Ўзбек, أۇزبېك', 'Uzbek'), ('ve', 'Tshivenḓa', 'Venda'),
#     ('vi', 'Tiếng Việt', 'Vietnamese'), ('vo', 'Volapük', 'Volapük'), ('wa', 'Walon', 'Walloon'),
#     ('cy', 'Cymraeg', 'Welsh'), ('wo', 'Wollof', 'Wolof'), ('fy', 'Frysk', 'Western Frisian'),
#     ('xh', 'isiXhosa', 'Xhosa'),
#     ('yi', 'ייִדיש', 'Yiddish'), ('yo', 'Yorùbá', 'Yoruba'), ('za', 'Saɯ cueŋƅ, Saw cuengh', 'Zhuang, Chuang'))



import uuid
from datetime import timedelta, datetime, timezone

import pytz
from django.conf import settings
from django.contrib.auth.models import BaseUserManager, AbstractBaseUser, PermissionsMixin
from django.core.mail import send_mail
from django.core.validators import validate_email
from django.db import models
from django.utils import timezone
from django.utils.translation import ugettext_lazy as _
from django_countries.fields import CountryField
from phonenumber_field.modelfields import PhoneNumberField

from django_user_email_extension.languages import LANGUAGES
from django_user_email_extension.validators import validate_users_min_age, validate_alphabetic_string


class Address(models.Model):
    TIMEZONES = tuple(zip(pytz.all_timezones, pytz.all_timezones))

    street_name = models.CharField(max_length=128, help_text='Street address, P.O. box, company name, c/o')
    street_number = models.CharField(max_length=64, help_text='Apartment, suite, unit, building, floor, etc.')
    city = models.CharField(max_length=64)
    state = models.CharField(max_length=64, null=True, blank=True)

    # uses https://github.com/SmileyChris/django-countries#countryfield
    country = CountryField()
    zip_code = models.IntegerField()
    timezone = models.CharField(max_length=32, choices=TIMEZONES, default='UTC')

    created_at = models.DateTimeField(_('Date Created'), auto_now_add=True, blank=True, editable=False)

    class Meta:
        unique_together = (('street_name', 'street_number', 'city', 'state', 'country'),)  # Set primary combined key
        verbose_name = _('address')
        verbose_name_plural = _('address')
        db_table = 'address'

    def __str__(self):
        if self.state:
            return '{} {} {}, {}, {}, {}'.format(self.street_name,
                                                 self.street_number,
                                                 self.city,
                                                 self.state,
                                                 self.zip_code,
                                                 self.country)
        return '{}, {}, {}, {}, {}'.format(self.street_name,
                                           self.street_number,
                                           self.city,
                                           self.zip_code,
                                           self.country)


class PhoneNumberManager(models.Manager):

    def get_all_phone_numbers_of_user(self, user):
        """
        return all phone numbers associated of user
        :param user: User object
        :return: QuerySet of PhoneNumber object, or empty
        """
        return self.all().filter(owner=user)

    def get_verified_phone_numbers_of_user(self, user):
        """
        return only verified phone numbers of user
        :param user: User object
        :return: QuerySet of PhoneNumber object(s), or empty
        """
        return self.all().filter(owner=user, verified=True)

    def get_verified_number_list(self, user):
        """
        return a list of verified numbers.

        return example [<PhoneNumber: +41524204242>, <PhoneNumber: +972535251234>]
        :param user: User object
        :return: list of PhoneNumberField
        """
        result = []
        for phone_object in self.get_verified_phone_numbers_of_user(user=user):
            result.append(phone_object.number)
        return result


class UserPhoneNumber(models.Model):
    # uses https://github.com/stefanfoulis/django-phonenumber-field
    number = PhoneNumberField()

    verified = models.BooleanField(
        _('Verified Status'),
        default=False,
        help_text=_('Define if this number have been verified.'),
    )
    __original_verified = None
    is_default = models.BooleanField(
        _('Default Status'),
        default=False,
        help_text=_('Define if this number is users default number.'),
    )
    owner = models.ForeignKey(settings.AUTH_USER_MODEL,
                              related_name='phone_number_obj',
                              on_delete=models.CASCADE)
    created_at = models.DateTimeField(_('Date Created'), auto_now_add=True, null=True, blank=True)
    verified_status_updated_at = models.DateTimeField(null=True, editable=False)

    objects = PhoneNumberManager()

    class Meta:
        db_table = 'user_phone_numbers'
        constraints = [
            models.UniqueConstraint(fields=['number', 'owner'], name='unique_users_phone_number')
        ]
        verbose_name = _('User Phone Numbers')
        verbose_name_plural = _('User Phone Numbers')

    def __init__(self, *args, **kwargs):
        super(UserPhoneNumber, self).__init__(*args, **kwargs)
        self.__original_verified = self.verified

    def save(self, *args, **kwargs):
        # check if there is a change is the verified field
        if self.verified != self.__original_verified:
            # update timestamp
            self.verified_status_updated_at = datetime.now(tz=timezone.utc)

        # check there is no other same number(s) (from another owner) which are also verified,
        # if there are any, set all of them to False, so there is a unique number with `verified=True`,
        # allow exactly single 'verified=True' value per 'number'
        if self.verified:
            UserPhoneNumber.objects.filter(number=self.number, verified=True).update(verified=False)

            # if current owner still does not have any default number (first time a number is saved),
            # this under verified if statement since only verified number should be able default.
            if not UserPhoneNumber.objects.filter(owner=self.owner, is_default=True):
                self.is_default = True

        # check are no multiple numbers, for the same owner (user) which are `is_default=True`,
        # if there are any, set all of them to False, so there is a unique number (per user) with `is_default=True`.
        # allow exactly single 'is_default=True' value per 'owner'
        if self.is_default:
            UserPhoneNumber.objects.filter(owner=self.owner, is_default=True).update(is_default=False)

        # save new verified state to __original_verified for future checks
        self.__original_verified = self.verified
        super(UserPhoneNumber, self).save(*args, **kwargs)

    def __str__(self):
        return str(self.number)

    def get_mobile_number_carrier(self):
        """
        get numbers mobile carrier, return empty string is not found.
        :return: string
        """
        from phonenumbers import carrier, parse
        number = parse(str(self.number))
        return carrier.name_for_number(number, "en")

    def get_number_location_description(self):
        """
        return a text description of a PhoneNumber object, return empty string is not found.
        :return: string
        """
        from phonenumbers import geocoder, parse
        return geocoder.description_for_number(parse(str(self.number)), "en")


class UserManager(BaseUserManager):
    def _create_user(self, email, password, **extra_fields):
        """
        Creates and saves a User with the given email and password.
        """
        if not email:
            raise ValueError('Email must be set')
        email = self.normalize_email(email)
        user = self.model(email=email, **extra_fields)
        user.set_password(password)
        user.save()
        return user

    def create_user(self, email, password=None, **extra_fields):
        extra_fields.setdefault('is_superuser', False)
        return self._create_user(email, password, **extra_fields)

    def create_superuser(self, email, password, **extra_fields):
        extra_fields.setdefault('is_staff', True)
        extra_fields.setdefault('is_superuser', True)
        extra_fields.setdefault('is_active', True)

        if extra_fields.get('is_staff') is not True:
            raise ValueError('Superuser must have is_staff=True.')
        if extra_fields.get('is_superuser') is not True:
            raise ValueError('Superuser must have is_superuser=True.')
        return self._create_user(email, password, **extra_fields)


class User(AbstractBaseUser, PermissionsMixin):
    email = models.EmailField(unique=True, primary_key=True, validators=[validate_email])
    user_name = models.CharField(_('User Name'), max_length=128, blank=True)
    first_name = models.CharField(_('First Name'), max_length=32, blank=True, validators=[validate_alphabetic_string])
    last_name = models.CharField(_('Last Name'), max_length=32, blank=True, validators=[validate_alphabetic_string])

    address = models.ManyToManyField(Address, verbose_name=_('Address'))

    USER_GENDER_CHOICES = (
        ('m', 'Male'),
        ('f', 'Female'),
        ('x', 'Not specified'),
    )
    gender = models.CharField(_('Gender'), choices=USER_GENDER_CHOICES, max_length=1, default='x')
    birth_date = models.DateField(_('Birth Date'), null=True, blank=True, validators=[validate_users_min_age])

    linkedin = models.URLField(max_length=255, blank=True)
    facebook = models.URLField(max_length=255, blank=True)
    github = models.URLField(max_length=255, blank=True)
    twitter = models.URLField(max_length=255, blank=True)

    registration_ip = models.GenericIPAddressField('Registered From', null=True)
    language = models.CharField(_('Users Language'), choices=LANGUAGES, max_length=2, default='en')

    is_staff = models.BooleanField(
        _('staff status'),
        default=False,
        help_text=_('Designates whether the user can log into this site.'),
    )
    is_active = models.BooleanField(
        _('Active Status'),
        default=False,
        help_text=_('Define if this user should be treated as active. '),
    )
    date_created = models.DateTimeField(_('Date Created'), auto_now_add=True, blank=True, editable=False)
    last_update_date = models.DateTimeField(auto_now=True, editable=False)
    last_login = models.DateTimeField(default=None, blank=True, null=True, editable=False)
    last_login_ip = models.GenericIPAddressField(_('Logged in from'), default=None, blank=True, null=True)

    objects = UserManager()

    USERNAME_FIELD = 'email'

    class Meta:
        verbose_name = _('user')
        verbose_name_plural = _('users')
        db_table = 'users'

    def __str__(self):
        return self.email

    def get_full_name(self):
        full_name = '%s %s' % (self.first_name, self.last_name)
        return full_name.strip()

    def get_first_name(self):
        return self.first_name

    def get_last_name(self):
        return self.last_name

    def get_user_addresses(self):
        return self.address.all()

    def get_all_phone_numbers(self):
        return self.phone_number_obj.get_all_phone_numbers_of_user(user=self)

    def get_verified_phone_numbers(self):
        return self.phone_number_obj.get_verified_phone_numbers_of_user(user=self)

    def get_verified_phone_numbers_list(self):
        return self.phone_number_obj.get_verified_number_list(user=self)

    def create_verification_email(self):
        self.email_verification_obj.create_verification(email=self.email)

    def send_verification_email(self, subject, body, from_mail):
        self.email_verification_obj.send_verification_email(subject,
                                                            body,
                                                            from_mail,
                                                            self.email)

    def get_uuid_of_email(self):
        return self.email_verification_obj.get_uuid_of_email(self.email)

    def get_default_phone_number(self):
        """
        :return: PhoneNumber object
        """
        return self.default_phone_number


class DjangoEmailVerifierManger(models.Manager):

    def create_verification(self, email, user=None):
        user = user or getattr(self, 'instance', None)
        if not user:
            raise ValueError('User object must be provided')

        if email not in user.email:
            raise ValueError('Email does not belong to user: %s', user.get_username())
        return self.create(user=user, email=email)

    # @receiver(post_save, sender=DjangoEmailVerifier, dispatch_uid="verify new account")
    # def send_verification_email(sender, instance, signal, *args, **kwargs):
    def send_verification_email(self, subject, body, from_mail, to_mail):
        email_verification_onj = self.all().get(email=to_mail)

        if not email_verification_onj.is_verified:
            send_mail(
                subject,
                body,
                from_mail,
                [email_verification_onj.email],  # To
                fail_silently=False,
            )

        return True

    def get_uuid_of_email(self, email):
        return self.all().get(email=email).verification_uuid


class DjangoEmailVerifier(models.Model):
    user = models.ForeignKey(settings.AUTH_USER_MODEL,
                             related_name='email_verification_obj',
                             on_delete=models.CASCADE)
    # user = models.OneToOneField(settings.AUTH_USER_MODEL, on_delete=models.CASCADE)
    email = models.EmailField(max_length=255)
    is_verified = models.BooleanField('verified', default=False)
    verification_uuid = models.UUIDField('Unique Verification UUID', default=uuid.uuid4)
    date_created = models.DateTimeField('Date Created', auto_now_add=True, blank=True)
    verified_at = models.DateTimeField(blank=True, null=True)

    objects = DjangoEmailVerifierManger()

    REQUIRED_FIELDS = ['user']

    class Meta:
        verbose_name = _('email verifications')
        verbose_name_plural = _('email verifications')
        db_table = 'email_verifications'

    def __str__(self):
        return 'Email Verification for User: ' + str(self.email)

    def verified(self):
        return self.is_verified

    def uuid_expire_date(self):
        # Only if DJANGO_EMAIL_VERIFIER_EXPIRE_TIME is not None,
        # Else, uuid never expire.
        # Set example: DJANGO_EMAIL_VERIFIER_EXPIRE_TIME=''
        hours_to_expire = getattr(settings, 'DJANGO_EMAIL_VERIFIER_EXPIRE_TIME', None)
        return self.date_created + timedelta(hours=hours_to_expire) if hours_to_expire is not None else None

    def is_uuid_expired(self):
        return timezone.now() >= self.uuid_expire_date()


def verify_record(uuid_value):
    # If input UUID exist in the db
    try:
        email_ver_object = DjangoEmailVerifier.objects.get(verification_uuid=uuid_value, is_verified=False)
    except DjangoEmailVerifier.DoesNotExist:
        raise Exception("Error - %s not associated to any email" % uuid_value)

    if email_ver_object.is_uuid_expired():
        raise Exception("Error - %s expired" % uuid_value)

    # If current object yet verified
    if not email_ver_object.verified():
        email_ver_object.is_verified = True
        email_ver_object.verified_at = timezone.now()
        email_ver_object.save()

    # Set account is active status
    email_ver_object.user.is_active = True
    email_ver_object.user.save()

    return True




from django.core.validators import RegexValidator
from django.core.exceptions import ValidationError
from django.conf import settings
from datetime import datetime


def phone_number_validator():
    return RegexValidator(regex=r'^\+?1?\d{12,15}$',
                          message="Phone number must be entered in the format of: '+18501234567'. "
                                  "Minimum 12 digits and up to 15 digits allowed.")


def validate_users_min_age(birth_date):
    if hasattr(settings, 'USER_MINIMAL_AGE'):
        minimal_age = int(settings.USER_MINIMAL_AGE)
        if minimal_age:
            from_date = datetime.now().date()
            try:
                x_years_ago = from_date.replace(year=from_date.year - minimal_age)
            except ValueError:  # 29th of february and not a leap year
                x_years_ago = from_date.replace(month=2, day=28, year=from_date.year - minimal_age)

            # in case birth date i bigger, then the date x years ago
            if birth_date > x_years_ago:
                raise ValidationError(message='Age must be at least {} years old.'.format(minimal_age))


def validate_alphabetic_string(string):
    """
    just use https://docs.python.org/3/library/stdtypes.html#str.isalpha
    :param string: string to validate
    :return: boolean id string is alphabetic
    """
    if not string.isalpha():
        raise ValidationError(message='Must be Alphabet string.')



from django.core.exceptions import ValidationError
from django.forms import Form, ModelForm, TextInput, Select, DateInput, NumberInput, CharField, RadioSelect, ChoiceField
from phonenumbers import is_valid_number, parse, NumberParseException, PhoneNumber

from django_user_email_extension.models import User, Address


class TokenForm(Form):
    token = CharField(
        widget=TextInput(attrs={'class': 'form-control'}))


class PhoneNumberVerificationForm(Form):
    """
    bootstrap ready form
    """
    country_code = CharField(widget=TextInput(attrs={'class': 'form-control'}),
                             required=True, help_text='Valid county code, eg +1')
    phone_number = CharField(widget=TextInput(attrs={'class': 'form-control'}))
    confirmation_type = ChoiceField(
        choices=[('sms', 'Send me a text')],
        # initial=sms will set checked radio button to 'sms' value
        widget=RadioSelect(attrs={'class': 'form-check-input'}), initial='sms', required=True)

    # override init function, so form will also be able to get a 'phone' key,
    def __init__(self, *args, **kwargs):
        # get PhoneNumberField object value passed to form if any
        phone_number_instance = kwargs.pop("phone", None)

        if isinstance(phone_number_instance, PhoneNumber):
            # set initial
            kwargs.update(initial={
                # 'field': 'value'
                'country_code': phone_number_instance.country_code,
                'phone_number': phone_number_instance.national_number,
            })

        super(PhoneNumberVerificationForm, self).__init__(*args, **kwargs)

    def clean_phone_number(self):
        phone_number = self.cleaned_data['phone_number']
        if not phone_number.isdigit():
            self.add_error('phone_number', 'Phone number should contain numbers only')
        return phone_number

    def clean_country_code(self):
        country_code = self.cleaned_data['country_code']
        if not country_code.startswith('+'):
            country_code = '+' + country_code
        return country_code

    def clean(self):
        data = self.cleaned_data

        phone_number = data['country_code'] + data['phone_number']
        try:
            phone_number = parse(phone_number, None)
            if not is_valid_number(phone_number):
                self.add_error('phone_number', 'Invalid phone number')
            else:
                # both values are actually cleaned here
                self.cleaned_data['country_code'] = phone_number.country_code
                self.cleaned_data['phone_number'] = phone_number
        except NumberParseException as e:
            self.add_error('phone_number', e)


class AddressForm(ModelForm):
    """
    bootstrap ready form
    """

    class Meta:
        model = Address
        exclude = ['timezone']
        widgets = {
            'street_name': TextInput(attrs={'class': 'form-control'}),
            'street_number': TextInput(attrs={'class': 'form-control'}),
            'city': TextInput(attrs={'class': 'form-control'}),
            'state': TextInput(attrs={'class': 'form-control'}),
            'country': Select(attrs={'class': 'form-control'}),
            'zip_code': NumberInput(attrs={'class': 'form-control'}),
        }

    def clean(self):
        # empty clean function so model will not block a user of attempt to add an already existing address
        return self.cleaned_data


class UserProfileForm(ModelForm):
    """
    bootstrap ready form
    """

    class Meta:
        model = User
        fields = ['first_name', 'last_name', 'gender', 'birth_date', 'language']
        widgets = {
            'first_name': TextInput(attrs={'class': 'form-control'}),
            'last_name': TextInput(attrs={'class': 'form-control'}),
            'gender': Select(attrs={'class': 'form-control'}),
            'birth_date': DateInput(attrs={
                'class': 'form-control datetimepicker-input',
                'data-target': '#datetimepicker1',
                'type': 'text'
            }),
            'language': Select(attrs={'class': 'form-control'}),
        }

    # # override init function, so it will also get a 'user' key,
    # def __init__(self, *args, **kwargs):
    #     super(UserProfileForm, self).__init__(*args, **kwargs)

    def clean_first_name(self):
        first_name = self.cleaned_data.get('first_name')
        if not first_name:
            raise ValidationError(message='First name cannot be empty')
        return first_name

    def clean_last_name(self):
        last_name = self.cleaned_data.get('last_name')
        if not last_name:
            raise ValidationError(message='Last name cannot be empty')
        return last_name

    def clean_birth_date(self):
        birth_date = self.cleaned_data.get('birth_date')
        if birth_date is None:
            raise ValidationError(message='Birth Date must be set')
        return birth_date



from django.contrib import admin
from django.contrib.auth.admin import UserAdmin
from django.utils.translation import ugettext_lazy as _

from django_user_email_extension.models import DjangoEmailVerifier, User, Address, UserPhoneNumber


@admin.register(User)
class UserAdmin(UserAdmin):
    fieldsets = (
        (None, {'fields': ('email', 'password')}),
        (_('Personal info'),
         {'fields': ('first_name', 'last_name', 'gender', 'birth_date', 'language')}),
        (_('Address info'), {'fields': ['address']}),
        (_('Social info'), {'fields': ('linkedin', 'facebook', 'github', 'twitter')}),
        (_('Permissions'), {'fields': ('is_active', 'is_staff', 'is_superuser', 'groups', 'user_permissions')}),
    )
    add_fieldsets = (
        (None, {
            'classes': ('wide',),
            'fields': ('email', 'first_name', 'last_name', 'password1', 'password2')}
         ),
    )
    list_display = ('email', 'first_name', 'last_name', 'is_active', 'is_staff')
    search_fields = ('email', 'first_name', 'last_name')
    ordering = ('email',)
    readonly_fields = ('date_created', 'last_update_date')
    pass


@admin.register(DjangoEmailVerifier)
class UserAdmin(admin.ModelAdmin):
    list_display = ('user', 'email', 'is_verified',)
    list_filter = ('is_verified',)
    ordering = ('user',)
    pass


@admin.register(Address)
class UserAdmin(admin.ModelAdmin):
    list_display = ('street_name', 'street_number', 'city', 'state', 'country', 'zip_code')
    list_filter = ('country',)
    ordering = ('country', 'city', 'street_name',)
    readonly_fields = ('created_at',)
    pass


@admin.register(UserPhoneNumber)
class UserAdmin(admin.ModelAdmin):
    list_display = ('number', 'owner', 'verified', 'is_default')
    list_filter = ('number', 'owner', 'verified', 'is_default')
    search_fields = ('number', 'owner',)
    ordering = ('number',)
    readonly_fields = ('created_at', 'verified_status_updated_at',)
    pass



from datetime import timedelta

from django.db.utils import IntegrityError
from django.test import TestCase

from django_user_email_extension.models import User, DjangoEmailVerifier, Address, UserPhoneNumber


class TestUserModel(TestCase):

    def setUp(self):
        self.user = User.objects.create_user(email="test@nalkins.cloud",
                                             user_name="arie",
                                             first_name="arie",
                                             last_name="lev",
                                             gender="m",
                                             linkedin="")

    def test_is_user(self):
        self.assertEqual(self.user.USERNAME_FIELD, "email")

    def test_get_full_name(self):
        self.assertEqual(self.user.get_full_name(), "arie lev")

    def test_get_first_name(self):
        self.assertEqual(self.user.get_first_name(), "arie")

    def test_get_last_name(self):
        self.assertEqual(self.user.get_last_name(), "lev")

    def test_create_verification_email(self):
        self.user.create_verification_email()
        email_ver_object = DjangoEmailVerifier.objects.get(email=self.user.email)

        self.assertEqual(email_ver_object.email, "test@nalkins.cloud")


class TestDjangoEmailVerifierModel(TestCase):

    def setUp(self):
        self.user = User.objects.create(email="test@nalkins.cloud")
        self.email_object = DjangoEmailVerifier.objects.create(user=self.user,
                                                               email=self.user.email)

    def test_to_string(self):
        """
        Test DjangoEmailVerifier __str__ function
        :return:
        """
        self.assertEqual(str(self.email_object), 'Email Verification for User: ' + self.user.email)

    def test_is_email_verification(self):
        """
        Test that required fields is 'user'
        :return:
        """
        self.assertEqual(self.email_object.REQUIRED_FIELDS, ['user'])

    def test_verified(self):
        """
        Test verified() function
        :return:
        """
        self.assertFalse(self.email_object.verified())
        self.email_object.is_verified = True
        self.assertTrue(self.email_object.verified())

    def test_uuid_expire_date(self):
        """
        Test email verification expiration,
        since test.setting.DJANGO_EMAIL_VERIFIER_EXPIRE_TIME value is 25, adding timedelta of 25 hours
        :return:
        """
        expected_expire = self.email_object.date_created.replace(microsecond=0,
                                                                 second=0,
                                                                 minute=0) + timedelta(hours=25)
        actual_expire = self.email_object.uuid_expire_date().replace(microsecond=0,
                                                                     second=0,
                                                                     minute=0)
        self.assertEqual(expected_expire, actual_expire)

    def test_is_uuid_expired(self):
        self.assertFalse(self.email_object.is_uuid_expired())


class TestUserManager(TestCase):

    def setUp(self):
        self.user = User.objects.create(email="test_user_manager@nalkins.cloud")
        DjangoEmailVerifier.objects.create_verification(email=self.user.email,
                                                        user=self.user)

    def test_create_user(self):
        User.objects.create(email="test_create_user@nalkins.cloud")

    def test_get_uuid_of_email(self):
        email_object = DjangoEmailVerifier.objects.get(email=self.user.email)
        uuid_num = self.user.email_verification_obj.get_uuid_of_email(email_object.email)
        self.assertRegex(str(uuid_num), '[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}')


class TestDjangoEmailVerifierManger(TestCase):

    def setUp(self):
        self.user = User.objects.create(email="test_verification_manager@nalkins.cloud")
        DjangoEmailVerifier.objects.create_verification(email=self.user.email, user=self.user)

    def test_create_verification(self):
        self.assertEqual(DjangoEmailVerifier.objects.get(email=self.user.email).email,
                         "test_verification_manager@nalkins.cloud")

    def test_get_uuid_of_email(self):
        self.email_object = DjangoEmailVerifier.objects.get(email=self.user.email)
        uuid_num = DjangoEmailVerifier.objects.get_uuid_of_email(self.email_object.email)
        self.assertRegex(str(uuid_num), '[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}')


class TestAddressModel(TestCase):
    def setUp(self):
        self.address_1 = Address.objects.create(street_name="98 Columbus Ave",
                                                street_number="Floor 5, Apartment 15",
                                                city="San Francisco",
                                                state="California",
                                                country='US',
                                                zip_code=123456)
        # apartment 24
        self.address_2 = Address.objects.create(street_name='441 Broadway St',
                                                street_number='Apartment 24',
                                                city='New York',
                                                state="NY",
                                                country='USA',
                                                zip_code=000000)
        # apartment 13
        self.address_3 = Address.objects.create(street_name='441 Broadway St',
                                                street_number='Apartment 13',
                                                city='New York',
                                                state="NY",
                                                country='USA',
                                                zip_code=000000,
                                                timezone='US/Eastern')

    def test_address_model(self):
        # should be 2 addresses with street '441 Broadway St'
        self.assertEqual(len(Address.objects.filter(street_name='441 Broadway St')), 2)

        # test string prints
        self.assertEqual(str(self.address_1),
                         '98 Columbus Ave Floor 5, Apartment 15 San Francisco, California, 123456, US')

    def test_unique_address(self):
        # This is should be single here due to this bug https://code.djangoproject.com/ticket/21540

        # test unique constraint ('street_name', 'street_number', 'city', 'state', 'country')
        with self.assertRaises(IntegrityError):
            Address.objects.create(street_name='441 Broadway St',
                                   street_number='Apartment 13',
                                   city='New York',
                                   state="NY",
                                   country='USA',
                                   zip_code=000000,
                                   timezone='US/Eastern')

    def test_country_field(self):
        # test country field https://github.com/SmileyChris/django-countries#countryfield

        country_field_object = Address.objects.filter(street_name='441 Broadway St',
                                                      street_number='Apartment 24').first().country
        self.assertEqual(country_field_object.name, 'United States of America')
        self.assertEqual(country_field_object.code, 'US')
        self.assertEqual(country_field_object.alpha3, 'USA')


class TestUserAddressInteraction(TestCase):
    def setUp(self):
        self.address_1 = Address.objects.create(street_name="98 Columbus Ave",
                                                street_number="Floor 5, Apartment 15",
                                                city="San Francisco",
                                                state="California",
                                                country='US',
                                                zip_code=123456)
        self.address_2 = Address.objects.create(street_name='441 Broadway St',
                                                street_number='Apartment 24',
                                                city='New York',
                                                state="NY",
                                                country='USA',
                                                zip_code=000000)
        self.user = User.objects.create_user(email="test_address@nalkins.cloud")
        self.user.address.add(self.address_1, self.address_2)

    def test_user_addresses(self):
        # current user should have 2 addresses
        self.assertEqual(len(self.user.address.all()), 2)


class TestPhoneNumberModel(TestCase):
    def setUp(self):
        self.user = User.objects.create_user(email="test_phone_number@nalkins.cloud")
        self.number_1 = UserPhoneNumber.objects.create(number='+1-212-509-5555', owner=self.user)

        self.number_1.full_clean()
        self.number_2 = UserPhoneNumber.objects.create(number='+972-50-123-4567', owner=self.user)
        self.number_2.full_clean()

    def test_model(self):
        # test get_mobile_number_carrier
        self.assertEqual('', self.number_1.get_mobile_number_carrier())
        self.assertEqual('Pelephone', self.number_2.get_mobile_number_carrier())

        # test get_number_location_description
        self.assertEqual('New York, NY', self.number_1.get_number_location_description())
